#! /bin/bash
# Copyright Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
#
# Linux版本视频流压测工具启动脚本
#
# 注意：如果需要在云手机服务端使用该工具，
#       请在config中修改绑核，并使用127.0.0.1访问本地云手机
#       以获得最佳性能
#
# 前置需求：已安装numactl
#
# 使用方法:
#
# 启动并连接一个或多个视频流客户端进程，可以在config中配置客户端绑核
# ./test.sh connect $ip $start_port $end_port
# ./test.sh connect $ip $port
#
# 断开一个或多个客户端进程
# ./test.sh disconnect $ip $start_port $end_port
# ./test.sh disconnect $ip $port
# 断开当前所有客户端
# ./test.sh disconnect all
#
# 打印指定客户端帧率和延时情况（可以在config里指定显示格式）
# ./test.sh display $ip $start_port $end_port
#
# 开始收集指定客户端帧率和延时情况（可以在config里指定显示格式）
# ./test.sh start $ip $start_port $end_port $run_time （单位：秒；最大值3小时）
#
# 查询当前已连接的客户端
# ./test.sh query
#
# 清理所有客户端进程和服务端进程（建议每完成一次测试后都执行，避免僵尸进程）
# ./test.sh clean
#
# 将screencap目录下从start_port到end_port的所有码流文件并行解码成jpg图片保存
# ./test.sh screencap $ip $start_port $end_port


#===============================================================================
# Variables
#===============================================================================
THISDIR=$(readlink -ef $(dirname $0))
CONFIG=$THISDIR/config
LOG_DIR=$THISDIR/log

#===============================================================================
# Functions
#===============================================================================
LOG_INFO() {
    echo -e "\033[1;36m[INFO] $@\033[0m"
}

LOG_WARN() {
    echo -e "\033[1;32m[WARNING] $@\033[0m"
}

LOG_ERROR() {
    echo -e "\033[1;31m[ERROR] $@\033[0m"
}

EXIT_ERROR() {
    LOG_ERROR "$@" ; exit 1
}

if [ "$(whoami)" != "root" ] ; then
    EXIT_ERROR "Please run this tool with root account!"
fi

chmod +x $THISDIR/cmdline
chmod +x $THISDIR/VideoClientHost
chmod +x $THISDIR/VideoEmulatorServer
export LD_LIBRARY_PATH=$THISDIR:$LD_LIBRARY_PATH

check_numactl() {
    cmd="numactl -s"
    $cmd >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        EXIT_ERROR "numactl unavailable, please install numactl \n
        On Ubuntu: apt install numactl \n
        On Euler : yum install numactl"
    fi
}

check_server() {
    # 检查server进程是否存在
    count=`ps -ef |grep VideoEmulatorServer |grep -v "grep" |wc -l`
    if [ $count == 0 ]; then
        nohup $THISDIR/VideoEmulatorServer >/dev/null 2>&1 &
        sleep 1
    fi
    count=`ps -ef |grep VideoEmulatorServer |grep -v "grep" |wc -l`
    if [ $count == 0 ]; then
        EXIT_ERROR "server not found, abort"
    fi
}

check_log_path() {
    if [ -f $LOG_DIR ]; then
        rm -r $LOG_DIR
    fi
    mkdir -p $LOG_DIR
}


main() {
    if [ ! -e "$CONFIG" ]; then
        EXIT_ERROR "Can not find file $CONFIG"
    fi
    # enable config
    source $CONFIG
    LOG_INFO "$CONFIG loaded"

    # 检查./screencap文件夹是否存在
    if [ ! -d "./screencap" ]; then
        mkdir "./screencap"
    fi

    if [ $1 != "clean" ]; then
        check_numactl
        #check_log_path
        check_server
    fi

    if [ $1 == "connect" ]; then
        ip=$2
        if [ $# == 3 ]; then
            port1=$3
            port2=$3
        elif [ $# == 4 ]; then
            port1=$3
            port2=$4
        else
            EXIT_ERROR "bad port num"
        fi

        for port in $(seq $port1 $port2); do
        if [ $port -gt 8999 ] || [ $port -lt 8001 ]; then
            EXIT_ERROR "bad port num $port"
        fi
        local numa=${NUMA_FOR_CLIENT[$((($port - 8001) % ${#NUMA_FOR_CLIENT[*]}))]}
        LOG_INFO "starting client $ip:$port on numa $numa"
        nohup numactl -N $numa $THISDIR/VideoClientHost $ip $port >/dev/null 2>&1 &
        sleep 0.5
        done
        LOG_INFO "checking current client num ..."
        sleep 1
        count==`ps -ef |grep VideoClientHost |grep -v "grep" |wc -l`
        LOG_INFO "current client process num: $count"

    elif [ $1 == "disconnect" ]; then
        ip=$2
        if [ $ip == 'all' ]; then
            $THISDIR/cmdline disconnect
        else
            if [ $# == 3 ]; then
                port1=$3
                port2=$3
            elif [ $# == 4 ]; then
                port1=$3
                port2=$4
            else
                EXIT_ERROR "bad port num"
            fi
            $THISDIR/cmdline disconnect $ip $port1 $port2
        fi
        LOG_INFO "checking current client num ..."
        sleep 3
        count==`ps -ef |grep VideoClientHost |grep -v "grep" |wc -l`
        LOG_INFO "current client process num: $count"

    elif [ $1 == "start" ]; then
        ip=$2
        port1=$3
        port2=$4
        time=$5
        $THISDIR/cmdline start $ip $port1 $port2 $time $INSTANCE_IN_ROW $NUMA_NUM \
            $COLLAPSE_NULL_CLIENT $AVG_FPS_STANDARD $AVG_FPS_THRESHOLD $AVG_FPS_THRESHOLD_PERCENT $MIN_FPS_LIMIT \
            $DISPLAY_LAG $DISPLAY_THRESHOLD_PERCENT $DISPLAY_MIN_FPS

    elif [ $1 == "display" ]; then
        ip=$2
        port1=$3
        port2=$4
        $THISDIR/cmdline display $ip $port1 $port2 $INSTANCE_IN_ROW $COLLAPSE_NULL_CLIENT \
            $AVG_FPS_STANDARD $AVG_FPS_THRESHOLD $DISPLAY_LAG

    elif [ $1 == "query" ]; then
        ps -ef |grep VideoClientHost |grep -v "grep" |awk '{print $(NF-1), $NF}'

    elif [ $1 == "clean" ]; then
        count=`ps -ef |grep VideoClientHost |grep -v "grep" |wc -l`
        if [ $count -ne 0 ]; then
            LOG_INFO "killing VideoClientHost"
            `ps -ef |grep VideoClientHost |grep -v "grep" |awk '{print $2}' |xargs kill -9`
        fi

        count=`ps -ef |grep VideoEmulatorServer |grep -v "grep" |wc -l`
        if [ $count -ne 0 ]; then
            LOG_INFO "killing VideoEmulatorServer"
            `ps -ef |grep VideoEmulatorServer |grep -v "grep" |awk '{print $2}' |xargs kill -9`
        fi
    elif [ $1 == "screencap" ]; then
        # 获取start和end参数
        ip=$2
        start=$3
        end=$4

        # 将IP地址转换为long类型的值
        IFS='.' read -r a b c d <<< "$ip"
        long_value=$((a*16777216 + b*65536 + c*256 + d))

        # 创建临时目录存放拷贝的文件
        temp_dir="./temp_screencap"
        mkdir -p $temp_dir

        # 遍历指定范围内的文件
        for ((i=start; i<=end; i++)); do
            # 查找以.h264和.265为后缀的文件
            for file in ./screencap/*; do
                if [[ "$file" == *"$long_valueAnd$i"* ]]; then
                    # 获取文件名
                    filename=$(basename "$file")
                    # 拷贝文件到临时目录
                    cp "$file" "$temp_dir/$filename"
                    # 更改文件权限为可读
                    chmod 644 "$temp_dir/$filename"
                    if [ -f "./screencap/${i}.jpg" ]; then
                        # 每次截图时自动刷新从start_port到end_port的画面
                        rm ./screencap/${i}.jpg
                    fi
                    # 使用ffmpeg解码文件为JPEG图片（并行处理）
                    ffmpeg -loglevel quiet -i "$temp_dir/$filename" "./screencap/${ip}_${i}.jpg" &
                fi
            done
        done

        # 等待所有后台任务完成
        wait

        # 删除临时目录中的拷贝文件
        rm -rf "$temp_dir"
        LOG_INFO "screencap finished"
    fi
}

main "$@"
